#define F_CPU 1000000UL  // CPU Frequency, this define is used by delay.h
                         //1 MHz is default for the internal clock for atMega88
#include <avr/delay.h>
#include <avr/io.h>

/** Program that produces 16.700.000 colors! 
 * Using an RGB-LED on PORTB. It will run on the
 * free AVR developmentkit from Rikard Strmmer, www.core.st
 * This program should be compiled with gcc-avr / win-avr
 *
 * This is just a sample program, no support, no nothing.
 * Other software is documented, this is just a hack.
 *
 * Shifting colors and intensity, like a perfect gem on a sunny day.
 * 
 * Test setup:
 * ATmega48 or ATmega88, or ATtiny13 or any appropriate Atmel AVR.
 * VCC = 5v during primary test, later 3v was used.
 * PORTB.0 Red   LED, 220 Ohm resistor in serie to VCC
 * PORTB.1 Green LED, 110 Ohm resistor in serie to VCC
 * PORTB.2 Blue  LED, 220 Ohm resistor in serie to VCC
 * PORTB.3 Green LED, 110 Ohm resistor in serie to VCC (Green was weak)
 * (Hi-intense Red, clear Green and Blue UV400nm led @ 3500mcd)
 * 
 * Edit the Makefile's MCU = <your MCU>, set F_CPU above to your CPU frequency,
 * make clean program
 *
 * @author Rikard Strmmer
 */

#define RED    1
#define GREEN  10  //2+8 The Green LED was weak, had to get two.
#define BLUE   4

int main(void)
{
	//set PORTB for output
  DDRB = 0xFF;

	unsigned char intensityUp = 0;//direction of intensity count
	unsigned char minIntensity = 2;
	
	unsigned char i = 0;
	//	unsigned char c = 0;
  unsigned char ledInit = 0;

	//the color
	unsigned char r = 255;
	unsigned char g = minIntensity;
	unsigned char b = minIntensity;

  

  while(1)
  {
		if( minIntensity == b)
		{
			if(( 255 == r ) && ( 255 != g)) //rgb 255,0,0 --> 255,255,0
				++g;
			else
			{
				if(( 255 == g ) && ( minIntensity < r ))// 255,255,min --> min,255,min
					--r;
				
				if( minIntensity == r)
				{
					if( 1 == intensityUp )
					{
						++minIntensity;
						if( 255 == minIntensity )
							intensityUp = 0;
					}
					else
					{
						--minIntensity;
						if( 0 == minIntensity )
							intensityUp = 1;
					}
					r = minIntensity;
					b = minIntensity;
				}
			}
		}		

		if( minIntensity == r )
		{
			if(( 255 == g ) && ( 255 != b))
				++b;
			else
				if(( 255 == b ) && ( minIntensity < g ))
					--g;
		}

		if( minIntensity == g )
		{
			if(( 255 == b ) && ( 255 != r))
				++r;
			else
				if(( 255 == r ) && ( minIntensity < b ))
					--b;
		}	
		
			//Don't light up any diodes if their intensidy is set to 0
			ledInit = 0; //0 on a pin means LED is litt.
			if( 0 == r ) ledInit +=RED;
			if( 0 == g ) ledInit +=GREEN;
			if( 0 == b ) ledInit +=BLUE;
			

			for(int d = 0; d < 20 ; d++)//extra loops, outher delay loop 20 to 100
			{
				i = 0;
				//Last thing we do before entering the loop is turning on the LED's
				PORTB=ledInit;
				do//my guess is that this loop is 12 clock cyckles long
				{
					++i;
					if( i == r) //turn off if intensity delivered
						PORTB |= RED;
					if( i == g)
						PORTB |= GREEN;
					if( i == b)	
						PORTB |= BLUE;				
				} while ( 255 != i);	
			}	
	}
	return 1;//this never happens
}
